#pragma once
#include "exceptions.h"
#include "_project.h"
#include "winlogger.h"

#include <boost/any.hpp>
#include <boost/fusion/container/generation/make_vector.hpp>
#include <boost/lambda/lambda.hpp>
#include <algorithm>

#include "ConnectionManager.h"
#include "libpq-fe.h"
#include "pqtypesimport.h"
#include "MultiArray.h"
#include "smartpqtypesutils.h"

#include <boost/ptr_container/ptr_array.hpp>

// Macros for checking errors in PGresult
#define CHECKSTATUS(connmanager, conn, result, errpoint) \
                try\
                {\
                Smartdb::CheckStatus(conn, result, errpoint);\
                }\
                catch(Smartdb::broken_connection)\
                {\
                connmanager->closeConnection();\
                throw;\
                }

#define CHECKEQ(left, right, errpoint) do{ \
                if((left) == (right)){ \
                throw Smartdb::failure(pqtypesimport::PQgeterror(), errpoint); \
                } \
}while(0)

//typedefs for f'tors
typedef std::map<std::string, boost::any> MapAny;
typedef std::vector<std::string> VecStr; 
typedef std::pair<std::size_t, int> PairSz; 
//print format properties
struct FormatCols
{
	const char fill_;
	const unsigned int width_;
	FormatCols(const char fill, unsigned int width)
		:
		fill_(fill),
		width_(width)
	{;}
};
typedef std::map<unsigned int, FormatCols> FormatColsMap;

class CDBExecutor;
namespace Detail
{
                using namespace PGTypeReg;

                struct free_pgarray_functor
                {
                                typedef PGarray element_type;

                                void operator()(PGarray *ptr)
                                {
                                                LibPQExp::_PQclear(ptr->res);

                                                PQparamReset(ptr->param);
                                                PQparamClear(ptr->param);

                                                //delete ptr->param;
                                                delete ptr;
                                }
                };
                struct free_pgparam
                {
                                typedef PGparam element_type;

                                void operator()(PGparam *ptr)
                                {
                                                //LibPQExp::_PQclear(ptr);
                                                PQparamReset(ptr);
                                                PQparamClear(ptr);

                                                delete ptr;
                                }
                };
                struct free_pgresult
                {
                                typedef PGresult element_type;

                                void operator()(PGresult *ptr)
                                {
                                                LibPQExp::_PQclear(ptr);
                                }
                };
                template<class T>
                class auto_free_ptr
                {
                                typedef typename T::element_type element_type;

                                element_type *ptr_;
                public:
                                auto_free_ptr(element_type *ptr)
                                                : ptr_(ptr)
                                {}
                                auto_free_ptr()
                                                : ptr_(NULL)
                                {}

                                element_type *operator->() { return ptr_; } 
                                element_type *get() { return ptr_; } 

                                element_type *release() { element_type *tmp = ptr_; ptr_ = NULL; return tmp; } 

                                void set(element_type *ptr) { ptr_ = ptr; } 

                                ~auto_free_ptr()
                                {
                                                if (ptr_)
                                                {
                                                                T()(ptr_);//call free function
                                                                ptr_ = NULL;
                                                }
                                }
                private:
                                auto_free_ptr(const auto_free_ptr&);
                                auto_free_ptr& operator=(const auto_free_ptr&);
                };

                class DBManager
                {
                public:
                                class QueryParams//todo rename
                                {
                                                PGparam *par_;
                                                PGconn *conn_;

                                public:
                                                class array_param//todo rename
                                                {
                                                                auto_free_ptr<free_pgarray_functor> arr;
                                                                std::string element_type_;
                                                                QueryParams *q_para_;

                                                public:
                                                                array_param(QueryParams *par, std::string &elementt)
                                                                                : q_para_(par), 
                                                                                  element_type_(elementt)
                                                                {
                                                                    std::auto_ptr<PGparam> arr_par(q_para_->create_param());
                                                                    if (!arr_par.get())
                                                                    {
																		throw "errro";//todo change error type
                                                                    }
                                                                    arr.set(new PGarray());
                                                                    arr->param = arr_par.release();
                                                                    arr->ndims = 0;
                                                                };
                                                                void set_num_dim(const size_t ndims) 
                                                                { 
																	arr->ndims = static_cast<int>(ndims);
                                                                }
                                                                void set_dims(const size_t *dims)
                                                                { 
                                                                    const size_t *st = dims;
                                                                    std::copy(st, st + arr->ndims, arr->dims);
                                                                    std::for_each<int*>(arr->lbound, arr->lbound + arr->ndims, boost::lambda::_1 = 1);
                                                                }
                                                                template<class T>
                                                                void add_element(T *el) 
                                                                {
																	q_para_->putData_(element_type_, el, arr->param);
                                                                }
                                                                void add_element(std::string *el) 
                                                                {
																	q_para_->putString(element_type_, el, arr->param);
                                                                }
                                                                void add_null_element() 
                                                                {
																	q_para_->put_null(arr->param);
                                                                }
                                                                void flush()
                                                                {
                                                                    std::string array_type = element_type_ + "[]";
                                                                    q_para_->putData(array_type.c_str(), arr.get());
                                                                }
                                                };
                                                QueryParams(PGparam *par, PGconn *conn)
                                                                : par_(par),
                                                                conn_(conn)
                                                {
                                                };
                                                size_t get_num_par() { return par_->vcnt; }

                                                PGparam *getParam() { return par_; }

                                                void putData(const std::string &colType, PGvarcharContainer *resData, PGparam *par, const boost::false_type&) const
                                                {
                                                    int r = PQputf( par, colType.c_str(), (char*)*resData);
                                                    CHECKEQ(r, 0, SOURCELINE);
                                                }
                                                template<class T>
                                                void put_value(const std::string &colType, T &resData) const
                                                {
                                                    int r = PQputf(par_, colType.c_str(), resData);
                                                    CHECKEQ(r, 0, SOURCELINE);
                                                }
                                                /*
                                                void put_value(const std::string &colType, T &resData) const
                                                {
                                                                struct
                                                                {
                                                                  int len;
                                                                  char *data;
                                                                } tmp;
                                                                tmp.len = 

                                                                int r = PQputf(par_, colType.c_str(), resData);
                                                                CHECKEQ(r, 0, SOURCELINE);

                                                                int r = PQputf( par, colType.c_str(), (char*)*resData);
                                                                CHECKEQ(r, 0, SOURCELINE);
                                                }*/
                                                
                                                void put_null()
                                                {
                                                    int r = PQputf(par_, "%null");
                                                    CHECKEQ(r, 0, SOURCELINE);
                                                }
                                                void put_null(PGparam *par)
                                                {
                                                    int r = PQputf(par, "%null");
                                                    CHECKEQ(r, 0, SOURCELINE);
                                                }
                                                template <class T>
                                                void putData(const std::string &colType, T *resData, PGparam *par, const boost::false_type&) const
                                                {
                                                    int r = PQputf( par, colType.c_str(), resData);
                                                    CHECKEQ(r, 0, SOURCELINE);
                                                }
                                                template <class T>
                                                void putData(const std::string &colType, T *resData, PGparam *par, const boost::true_type&) const
                                                {
													int r = PQputf( par, colType.c_str(), *resData);
                                                    CHECKEQ(r, 0, SOURCELINE);
                                                }
                                                template<class T>
                                                void putData_(const std::string &colType, T *resData, PGparam *par)
                                                {
                                                    if (resData == NULL)
                                                    {
                                                        int r = PQputf(getParam(), "%null", 0);//todo remove %null
                                                        CHECKEQ(r, 0, SOURCELINE);
                                                        return;
                                                    }
                                                    else
                                                    {
                                                        putData(colType, resData, par, boost::mpl::if_<boost::mpl::and_< boost::is_arithmetic<T>, boost::mpl::not_<boost::is_pointer<T>>>, boost::true_type,boost::false_type>::type());
                                                    }
                                                }
                                                array_param new_array_param(std::string &col_type)
                                                {
                                                    return array_param(this, col_type);
                                                }

                                                template<class T>
                                                void putData(const std::string &colType, T *resData)
                                                {
                                                    putData_(colType, resData, getParam());
                                                }
                                                void putString(const std::string &colType, const std::string *resData)
                                                {
													putString(colType, resData, getParam());
                                                }
                                                void putString(const std::string &colType, const std::string *resData, PGparam *param)
                                                {
                                                    int r = PQputf(param, colType.c_str(), resData->c_str());
                                                    CHECKEQ(r, 0, SOURCELINE);
                                                }
                                protected:
                                                PGparam *create_param() { return pqtypesimport::PQparamCreate(conn_); }

                                };

                                class QueryResults
                                {
                                public:
                                                QueryResults(PGresult *res)
                                                                : res_(res)
                                                                , currentRow_(0)
                                                {
                                                                num_col_ = retrieveNumCols();
                                                }
                                                ~QueryResults()
                                                {
                                                                if(getResult())
                                                                {
                                                                                LibPQExp::_PQclear(getResult());//fix it
                                                                                //PQclear(getResult());
                                                                                getResult() = NULL;
                                                                }
                                                }
                                                void resetToFirstRow()
                                                {
                                                                currentRow_ = 0;
                                                }
                                                unsigned int getNumCols() const
                                                {
                                                                return num_col_;
                                                }
                                                unsigned int retrieveNumCols() const
                                                {
                                                                return LibPQExp::_PQnfields(res_);
                                                }
                                                unsigned int getNumRows() const 
                                                { 
                                                                return LibPQExp::_PQntuples(res_);
                                                }
                                                void nextRow()
                                                { 
                                                                if (currentRow_ < getNumRows())
                                                                                ++currentRow_;
                                                                else ;/*todo throw execption*/
                                                }; 
                                                bool hasMoreRows() const { return currentRow_ < getNumRows(); };

                                                bool isNull(const unsigned int col)
                                                {
                                                                return LibPQExp::_PQgetisnull(getResult(), currentRow_, col) == 1;
                                                }

                                                template<class T, template<class> class Arr>
                                                void getArray(const unsigned int col, const std::string &arrType, Arr<T>* &resData)
                                                {
                                                                auto_free_ptr<free_pgarray_functor> arr(new PGarray());

                                                                int r = PQgetf( getResult(), 
                                                                                currentRow_, 
                                                                                arrType.c_str(), 
                                                                                col, 
                                                                                arr.get()
                                                                                );
                                                                CHECKEQ(r, 0, SOURCELINE);
                                                                //todo add checking for r
                                                                

                                                                resData->resize_it<int*>((int*)arr->dims, (int*)arr->dims + arr->ndims);

                                                                int *it = (int *)arr->dims, 
                                                                                *itend = (int *)arr->dims + arr->ndims;
                                                                size_t sum = 1;
                                                                for(; it != itend; ++it)
                                                                {
                                                                                sum *= *it;
                                                                }
                                                                std::string elementType = arrType.substr(0, arrType.length() - 2);//todo check for good format

                                                                typedef typename Arr<T>::iterator iterator;
                                                                iterator arrIt = resData->begin();
                                                                for(int i = 0; arrIt != resData->end() && i < sum; ++arrIt, ++i)
                                                                {
                                                                                if (LibPQExp::_PQgetisnull(arr->res, i, 0))
                                                                                {
                                                                                                resData->replace(arrIt, NULL);
                                                                                                continue;
                                                                                }
                                                                                r = pqtypesimport::PQgetf(arr.get()->res, i , elementType.c_str(), 0, &(*arrIt));//todo check for null
                                                                                CHECKEQ(r, 0, SOURCELINE);

                                                                }
                                                }
                                                void getArray(const unsigned int col, const std::string &arrType, multi_array<std::string>* &resData)
                                                {
                                                                auto_free_ptr<free_pgarray_functor> arr(new PGarray());

                                                                int r = PQgetf( getResult(), 
                                                                                currentRow_, 
                                                                                arrType.c_str(), 
                                                                                col, 
                                                                                arr.get()
                                                                                );
                                                                CHECKEQ(r, 0, SOURCELINE);
                                                                //todo add checking for r

                                                                resData->resize_it<int*>((int*)arr->dims, (int*)arr->dims + arr->ndims);

                                                                int *it = (int *)arr->dims, 
                                                                                *itend = (int *)arr->dims + arr->ndims;
                                                                int sum = 1;
                                                                for(; it != itend; ++it)
                                                                {
                                                                                sum *= *it;
                                                                }
                                                                std::string elementType = arrType.substr(0, arrType.length() - 2);//todo check for good format

                                                                typedef multi_array<PGvarcharContainer>::iterator iterator;
                                                                iterator arrIt = resData->begin();
                                                                for(int i = 0; arrIt != resData->end() && i < sum; ++arrIt, ++i)
                                                                {
                                                                                if (LibPQExp::_PQgetisnull(arr->res, i, 0))
                                                                                {
                                                                                                resData->replace(arrIt, NULL);
                                                                                                continue;
                                                                                }

                                                                                char *temp;
                                                                                int r = pqtypesimport::PQgetf(arr.get()->res, i , elementType.c_str(), 0, &temp);
                                                                                CHECKEQ(r, 0, SOURCELINE);//todo maka exeception safe

                                                                                *arrIt = temp;
                                                                }
                                                }
                                                void getString(const unsigned int col, const std::string &colType, std::string &resData)
                                                {
                                                                char *res = NULL;
                                                                int r = pqtypesimport::PQgetf( getResult(),
                                                                                currentRow_,
                                                                                colType.c_str(),
                                                                                col,
                                                                                &res
                                                                                );
                                                                CHECKEQ(r, 0, SOURCELINE);
                                                                resData = res;
                                                }
                                                void getBytea(const unsigned int col, const std::string &colType, PGbytea &resData)
                                                {
                                                                struct _PGbytea{
                                                                                int len;
                                                                                char* data;
                                                                } data;
                                                                char *res = NULL;
                                                                int r = pqtypesimport::PQgetf( getResult(),
                                                                                currentRow_,
                                                                                colType.c_str(),
                                                                                col,
                                                                                &data
                                                                                );
                                                                CHECKEQ(r, 0, SOURCELINE);
                                                                resData = PGbytea(data.len, data.data);
                                                }

                                                template<class T>
                                                void getData(const unsigned int col, const std::string &colType, T* &resData)
                                                {
                                                                int r = pqtypesimport::PQgetf( getResult(),
                                                                                currentRow_,
                                                                                colType.c_str(),
                                                                                col,
                                                                                resData
                                                                                );
                                                                CHECKEQ(r, 0, SOURCELINE);
                                                }
                                private:
                                                inline PGresult *&getResult(){ return res_; };

                                                void clearResults() 
                                                {
                                                                if(getResult())
                                                                {
                                                                                PQclear(getResult());
                                                                                getResult() = NULL;
                                                                }
                                                } 
                                                unsigned int currentRow_;
                                                unsigned int num_col_;
                                                PGresult *res_;
                                };

                                DBManager(ConnectionManager &c)
                                                : connManager_(c),
                                                conn_(NULL)
                                {
                                };

                                void prepareConnection()
                                {
                                                conn_ = connManager_.getConnection();
                                };
                                QueryParams *getQueryParam()
                                {
                                                prepareConnection();

                                                PGparam *param = pqtypesimport::PQparamCreate(conn_);
                                                if (!param)
                                                {
                                                                throw "error";//todo add appropriate exeception
                                                }
                                                QueryParams *res = new QueryParams(param, conn_);
                                                return res;
                                }

                                QueryResults execParamPrepared(QueryParams& params, const std::string &query_key, int format)
                                {
                                    prepareConnection();
								
									PGresult *result = pqtypesimport::PQparamExecPrepared(conn_, params.getParam(), query_key.c_str(), format);
                                    auto_free_ptr<free_pgresult> res(result);

                                    CHECKSTATUS((&connManager_),conn_, result, SOURCELINE);

                                    return QueryResults(res.release());
                                }
                                QueryResults execPrepared(const std::string &query_key, int format)
                                {
                                    prepareConnection();
									
                                    PGresult *result = pqtypesimport::PQparamExecPrepared(conn_, NULL, query_key.c_str(), format);
                                    auto_free_ptr<free_pgresult> res(result);

                                    CHECKSTATUS((&connManager_),conn_, result, SOURCELINE);

                                    return QueryResults(res.release());
                                }
                                void execParamQuery(QueryParams& params, const std::string &query, int format, bool esc = false)
                                {
                                    prepareConnection();
									std::string q = esc ? escapeQueryString(query) : query;
                                    //params.getParam()
                                    PGresult *result = NULL;
                                    result = pqtypesimport::PQparamExec(conn_, params.getParam(), q.c_str(), format);

                                    auto_free_ptr<free_pgresult> res(result);

                                    CHECKSTATUS((&connManager_),conn_, result, SOURCELINE);
                                    //todo checkings
                                }
                                void execSimpleQuery(const std::string &query, int format, bool esc = false)
                                {
                                    prepareConnection();
                                    std::string q = esc ? escapeQueryString(query) : query;
                                    PGresult *result = NULL;
                                    result = pqtypesimport::PQparamExec(conn_, NULL, q.c_str(), format);

                                    auto_free_ptr<free_pgresult> res(result);

                                    CHECKSTATUS((&connManager_),conn_, result, SOURCELINE);
                                    //todo checkings
                                }
                                void prepareQuery(const std::string &prep_key, const std::string &sql_cmd, size_t num_par, bool esc = false)
                                {
                                    prepareConnection();
									std::string q = esc ? escapeQueryString(sql_cmd) : sql_cmd;
                                    PGresult *result = LibPQExp::_PQprepare(conn_, prep_key.c_str(), q.c_str(), static_cast<int>(num_par), NULL);
                                    auto_free_ptr<free_pgresult> res(result);

                                    CHECKSTATUS((&connManager_),conn_, result, SOURCELINE);
                                };

                                QueryResults execQuery(QueryParams& params, const std::string &query, int format, bool esc = false)
                                {
                                    prepareConnection();
                                    std::string q = esc ? escapeQueryString(query) : query;
                                    PGresult *result = NULL;
                                    result = pqtypesimport::PQparamExec(conn_, params.getParam(), q.c_str(), format);
                                    auto_free_ptr<free_pgresult> res(result);

                                    CHECKSTATUS((&connManager_),conn_, result, SOURCELINE);

                                    return QueryResults(res.release());
                                }

                                QueryResults execQuery(const std::string &query, int format, bool esc = false)
                                {
                                    PGresult *result = NULL;
                                    prepareConnection();
									std::string q = esc ? escapeQueryString(query) : query;
                                    result = pqtypesimport::PQparamExec(conn_, NULL, q.c_str(), format);//TODO: change to PQexec
                                    auto_free_ptr<free_pgresult> res(result);

                                    CHECKSTATUS((&connManager_),conn_, result, SOURCELINE);

                                    return QueryResults(res.release());
                                };
                private:
					std::string escapeQueryString(const std::string& query) const
					{
						int err = 0;
						std::string result;
						boost::scoped_array<char> escapedQuery(new char[2*query.size()+1]);

						boost::scoped_array<char> from (new char[query.size()+1]);
						std::copy(query.begin(), query.end(), from.get());
						from[query.size()] = '\0'; //do not forget about the terminating '\0'

						size_t r = LibPQExp::_PQescapeStringConn(conn_, escapedQuery.get(), from.get(), strlen(from.get()), &err);
						if (err)
							throw Smartdb::argument_error(pqtypesimport::PQgeterror(), SOURCELINE);

						result = escapedQuery.get();
						return result;
					}

                    PGparam *&getParams() { return params_; };
                    void clearParams()
                    {
                        if (getParams())
                        {
                            PQparamReset(getParams());
                            PQparamClear(getParams());
                            getParams() = NULL;
                        }
                    }
                    ConnectionManager &connManager_;
                    int format_;
                    PGparam *params_;
                    PGconn *conn_;
                };

                struct getCell
                {
                    DBManager::QueryResults &results_;
                    unsigned int &col_;
                    std::vector<std::string> &colTypes_;

                    getCell(DBManager::QueryResults &results, unsigned int &col, std::vector<std::string> &colTypes)
                                    : results_(results),
                                    col_(col),
                                    colTypes_(colTypes)
                    {}
                    template <class T>
                    void operator()(multi_array<T> *&val) const
                    {
                        if (col_ >= results_.getNumCols())
                            return;
                        if (results_.isNull(col_))
                        {
                            delete val;
                            val = NULL;
                        }
                        else
                        {
							results_.getArray(col_, colTypes_[col_], val);
                        }
                        ++col_;
                        //             getData(
                    };
                    template <class T>
                    void operator()(T *&val) const
                    {
                        if (col_ >= results_.getNumCols())
							return;
                        if (results_.isNull(col_))
                        {
                            delete val;
                            val = NULL;
                        }
                        else
                        {
							results_.getData(col_, colTypes_[col_], val);
                        }
                        ++col_;
                        //             getData(
                    };

                    template <>
                    void operator()(PGvarchar *&val) const
                    {
                        if (col_ >= results_.getNumCols())
							return;
                        if (results_.isNull(col_))
                        {
                            delete val;
                            val = NULL;
                        }
                        else
                        {
							results_.getString(col_, colTypes_[col_], *val);
                        }
                        ++col_;
                    }
                    template <>
                    void operator()(PGbytea *&val) const
                    {
                        if (col_ >= results_.getNumCols())
							return;
                        if (results_.isNull(col_))
                        {
                            delete val;
                            val = NULL;
                        }
                        else
                        {
                            results_.getBytea(col_, colTypes_[col_], *val);
                        }
                        ++col_;
                    }
                };
                struct putCell
                {
                    DBManager::QueryParams &params_;
                    unsigned int &col_;
                    std::vector<std::string> &colTypes_;

                    putCell(DBManager::QueryParams &params, unsigned int &col, std::vector<std::string> &colTypes)
                                    : params_(params),
                                    col_(col),
                                    colTypes_(colTypes)
                    {}
                    template <class T>
                    void operator()(multi_array<T> *&in_array) const
                    {
                        if (in_array == NULL)
                        {
							params_.put_null();
                        }
                        else
                        {
                            std::string &arrayType = colTypes_[col_];
                            std::string elementType = arrayType.substr(0, arrayType.length() - 2);//todo check for good format

                            DBManager::QueryParams::array_param &arr_par = params_.new_array_param(elementType);

                            arr_par.set_num_dim(in_array->get_num_dims());

                            boost::scoped_array<std::size_t> dims(new std::size_t[in_array->get_num_dims()]);
                            in_array->get_dims<size_t*>(dims.get());

                            arr_par.set_dims(dims.get());

                            multi_array<T>::iterator i = in_array->begin();
                            for( ; i != in_array->end(); ++i)
                            {
                                if (multi_array<T>::is_null(i))
                                {
                                    arr_par.add_null_element();
                                    continue;
                                }
                                arr_par.add_element(&*i);
                            }
                            arr_par.flush();
                        }
                        ++col_;
                    };

                    template <class T>
                    void operator()(T *&val) const
                    {
                        if (val == NULL)
                        {
							params_.put_null();//TODO: remove magic string "%null"s
                        }
                        else
                        {
							params_.putData(colTypes_[col_], val);
                        }
                        ++col_;
                        //             getData(
                    };
                    template <>
                    void operator()(PGvarchar *&val) const
                    {
                        if (val == NULL)
                        {
							params_.put_null();//todo remove magic string "%null"s
                        }
                        else
                        {
                            params_.putString(colTypes_[col_], val);//todo modify it
                        }
                        ++col_;
                    }
                };
				struct printCell
				{
					//std::stringbuf *buf_;
					std::ostream &os_;
					unsigned int &col_;
					const char* delim_;
					const std::vector<std::string> &colTypes_;
					const FormatColsMap &colsFormat_;

					printCell(std::ostream &os, unsigned int &col, const std::vector<std::string> &colTypes, const FormatColsMap &colsFormat, const char* delim = "|")
						:
						os_(os),
						col_(col),
						colTypes_(colTypes),
						colsFormat_(colsFormat),
						delim_(delim)
					{;}
					//template <class T>
					//std::ostream& operator <<(std::ostream& os, multi_array<T> *&val) const
					//{
						//if (val->is_null())
						//{
						//	os << "{NULL}";
						//}
						//else
						//{
						//	//results_.getArray(col_, colTypes_[col_], val);
						//}
						//++col_;
						//return os;
					//};
					template <class V>
					void operator ()(V *val) const
					{
						FormatColsMap::const_iterator coln_it_ = this->colsFormat_.find(this->col_);
						if (coln_it_ != this->colsFormat_.end())
						{
							this->os_ << std::setiosflags(std::ios::right) << std::setfill((*coln_it_).second.fill_) << std::setw((*coln_it_).second.width_);
							this->os_ << *val;
							if (this->colsFormat_.size() != ++col_)
								this->os_ << this->delim_;
						}
						else 
							throw Smartdb::usage_error(boost::str(boost::format("No format options found for the column \"%1%\" ") % this->col_), SOURCELINE);
					};
					template <>
					void operator()(PGvarchar *val) const
					{
						//if (col_ >= results_.getNumCols())
						//	return;
						//if (results_.isNull(col_))
						//{
						//	delete val;
						//	val = NULL;
						//}
						//else
						//{
						//	results_.getString(col_, colTypes_[col_], *val);
						//}
						//this->os_ << *val << this->delim_;
						//this->buf_->sputn(val.c_str(), val.size());
						//this->buf_->sputc(this->delim_);
						FormatColsMap::const_iterator coln_it_ = this->colsFormat_.find(this->col_);
						if (coln_it_ != this->colsFormat_.end())
						{
							this->os_ << std::setiosflags(std::ios::left) << std::setfill((*coln_it_).second.fill_) << std::setw((*coln_it_).second.width_);
							this->os_ << *val;
							if (this->colsFormat_.size() != ++col_)
								this->os_ << this->delim_;
						}
						else 
							throw Smartdb::usage_error(boost::str(boost::format("No format options found for the column \"%1%\" ") % this->col_), SOURCELINE);
					};
					//template <>
					//void operator()(PGbytea *&val) const
					//{
					//	if (col_ >= results_.getNumCols())
					//		return;
					//	if (results_.isNull(col_))
					//	{
					//		delete val;
					//		val = NULL;
					//	}
					//	else
					//	{
					//		results_.getBytea(col_, colTypes_[col_], *val);
					//	}
					//	++col_;
					//}
				};

} 
//end of namespace Detail
