#include "stdafx.h"
#include "ConnectionManager.h"
#include "exceptions.h"


ConnectionManager::TLongLong ConnectionManager::val_ = 0;
NoticeDispatcher::TPtrContainer NoticeDispatcher::elements_ = std::map<std::string, UsrNoticeReceiver>();

PGconn* ConnectionManager::getConnection()
{
	if (autoReconnect_)
	{
		if (!conn_ || PQstatus(conn_) != CONNECTION_OK)
		{
			Connect();
			return conn_;
		}
	}
	else
	{
		if (!conn_)
		{
			throw Smartdb::broken_connection("Automatic reconnect is disabled", SOURCELINE);
		}
	}
	return conn_;
}
void ConnectionManager::Connect()
{
	//PQaddGlobalObjectHooks(pqtypesimport::PQtypesObjectHooks()); - commented by Nikolay while migrating from 1.0c version to 1.3.1

	conn_ = PQconnectdb(connInfo_.c_str());

	checkConnection();
	
	#ifdef _DO_SHOW_CONNECTION_DETAILS
	logger_->Log(logTRACE) << "Successfully connected to: " << conninfo;
	#endif
	if (conn_)
		PQsetNoticeReceiver(conn_, &NoticeDispatcher::noticeReceiver, (void *) this);
}
void ConnectionManager::getUniquePrepStmtName( /*out*/ std::string &newStmtName)
{
	std::stringstream tmp;
	std::string stmtName;
	
	tmp << val_;
	++val_;

	stmtName = UNIQUE_PREP_PREFIX + tmp.str();
	newStmtName = stmtName;

	if (!val_)
		logger_->Log(logWARNING) << "ConnectionManager::GetUniquePrepStmtName reset prepared statement counter to 0" ;
};
void ConnectionManager::closeConnection()
{
	if (conn_)
	{
		PQfinish(conn_);
		conn_ = NULL;
	}
	
};
void ConnectionManager::checkConnection()
{
	if(PQstatus(conn_) != CONNECTION_OK)
	{
		std::string err = boost::str(boost::format("connection failure: %1%\n") % PQerrorMessage(conn_));
		throw Smartdb::broken_connection(err, SOURCELINE);
	}
	if(!LibPQExp::_PQtypesRegister(conn_))
	{
		PQfinish(conn_);
		throw Smartdb::broken_connection("Failed to register libpqtypes as a libpq event proc", SOURCELINE);
	} 
}