
#include "stdafx.h"
#include "boost/test/debug.hpp"

#include <limits>
#include "buildintypes.h"
#include "arraytypes.h"
#include "link_.h"
#include "inst_.h"
#include "net_test_.h"
#include "link2_.h"
#include "asset_tradable_combination_.h"
#include "instruments_in_inst_vec_.h"

#include "winlogger.h"
#include "CDBExecutor.h"
#include "SmartStringUtils.h"


extern std::string connectionInfo;
static boost::shared_ptr<WinLogger> g_logger;
static boost::shared_ptr<CDBExecutor> dbExec;
static boost::shared_ptr<CDBExecutor> dbExec2;
static boost::shared_ptr<CDBExecutor> dbExec3;
static boost::shared_ptr<ConnectionManager> conn_;

static int SafeInitLoger()
{
	std::string str = "err", defName;
	std::string logLevel = "TRACE";
	std::string append = "NO";//generated file will be cleared on open

	std::string tm = DateTimeUtils::currentDate();
	str = ".\\" + str + tm + ".log";
	defName = "errors_" + tm + ".log";
	g_logger = WinLogger::Init(str, (!append.compare("YES")) ? false : true, logLevel, defName);
	g_logger->Log(logTRACE) << "Logger initialized successfully";
	return 0;
}

static WinLogger *InitLoger()
{
	if (g_logger.get())
		return g_logger.get();
	SafeInitLoger();
	return g_logger.get();
}
struct GlobalSetup {

	GlobalSetup()
	{ 
#ifdef IGNORE_MEMORY_LEAKS_DETECT
		boost::debug::detect_memory_leaks(false);
#endif
		InitLoger();
		conn_.reset(new ConnectionManager(connectionInfo, g_logger.get()));
		dbExec.reset( new CDBExecutor(conn_.get(), g_logger.get()));
		dbExec2.reset( new CDBExecutor(conn_.get(), g_logger.get()));
		dbExec3.reset( new CDBExecutor(conn_.get(), g_logger.get()));
	}
	~GlobalSetup()
	{
	}

};

BOOST_GLOBAL_FIXTURE( GlobalSetup );

struct InstTableSetup {

	inst_table testTbl;
	void dropOldTableInstanceCmd()
	{
		dbExec->execSQL("DROP TABLE IF EXISTS test.inst");
	}

	InstTableSetup()
	{ 
		dropOldTableInstanceCmd();

		std::string createTableCmd = "CREATE TABLE test.inst"
			"("
			"inst_id integer NOT NULL,"
			"quoting_place_id integer NOT NULL,"
			"asset_id integer NOT NULL,"
			"quoting_currency_id integer NOT NULL,"
			"quoting_currency_multiplier double precision,"
			"description character varying,"
			"time timestamp"
			");";
		dbExec->execSQL(createTableCmd.c_str());

		std::string insertIntoTableCmd = "INSERT INTO test.inst "
			"VALUES ($1,2,3,4,5,'test', '2001-09-27 23:00:00')";

		dbExec->addParam(PG_INT4, 1);
		dbExec->execSQL(insertIntoTableCmd);

		dbExec->addParam(PG_INT4, 2);
		dbExec->execSQL(insertIntoTableCmd);

		dbExec->addParam(PG_INT4, 2);
		dbExec->execSQL(insertIntoTableCmd);

	}
	~InstTableSetup()  
	{
		dropOldTableInstanceCmd();
	}
};

BOOST_AUTO_TEST_SUITE( SimpleCDBExecutorTests )

BOOST_AUTO_TEST_CASE( fastLinkTwoColumnsTest )
{
	int d[][4] = {{9,3, 4,8}, {8,4, 5,7}, {7,5, 7,5}, {6,6, 3,9},{5,7, 9,3},{4,9, 6,6},{4,8, 6,6},{5,5, 4,9},{3,9, 8,4}};

	new_buildin_table b, b1;

	for(int k  = 0; k < 9; ++k)
	{
		b.push_back(boost::shared_ptr<new_buildin_table::row_type>(new new_buildin_table::row_type));//reserved one row
		b1.push_back(boost::shared_ptr<new_buildin_table::row_type>(new new_buildin_table::row_type));//reserved one row

		b.back().set_colint2(d[k][0]);
		b.back().set_colint4(d[k][1]);

		b1.back().set_colint2(d[k][2]);
		b1.back().set_colint4(d[k][3]);
	}
	g_logger->Log(logTRACE) << "Link start";

	typedef external_link_cnt<new_buildin_table::row_type, new_buildin_table::row_type> link_res;
	link_res linkres;
	dbExec->linkRows<1,2,1,2>(b, b1, linkres);

	g_logger->Log(logTRACE) << "Link stop";

	link_res::iterator i = linkres.begin();

	unsigned sum = 0;
	for(; i != linkres.end(); ++i)
	{
		link_res::internal_link_cnt &i_cnt = (*i).second;

		link_res::internal_link_cnt::iterator j = i_cnt.begin();
		for(;j != i_cnt.end(); ++j)
		{
			sum += 1;
		}
	}
	BOOST_CHECK_EQUAL(sum, 9);
}

BOOST_AUTO_TEST_CASE( fastLinkSpeedTest )
{
	new_buildin_table b, b1;

	const int maxrows = 10;

	for(int i = 0; i < maxrows; ++i)
	{
		b.push_back(boost::shared_ptr<new_buildin_table::row_type>(new new_buildin_table::row_type));//reserved one row
		b1.push_back(boost::shared_ptr<new_buildin_table::row_type>(new new_buildin_table::row_type));//reserved one row

		b.back().set_colint2(i % 2);
		b1.back().set_colint2(i % 2);
	}

	g_logger->Log(logTRACE) << "Link start";

	typedef external_link_cnt<new_buildin_table::row_type, new_buildin_table::row_type> link_res;
	link_res linkres;
	dbExec->linkRows<1,1>(b, b1, linkres);

	g_logger->Log(logTRACE) << "Link stop";

	unsigned sum = 0;
	link_res::iterator i = linkres.begin();
	for(; i != linkres.end(); ++i)
	{
		link_res::internal_link_cnt &i_cnt = (*i).second;

		link_res::internal_link_cnt::iterator j = i_cnt.begin();
		for(;j != i_cnt.end(); ++j)
		{
			sum += 1;
		}
	}
	BOOST_CHECK_EQUAL(sum, 50);
}

BOOST_AUTO_TEST_CASE( VoidTypeTest )
{
	bool pass;
	std::string sStmtCmd = "CREATE OR REPLACE FUNCTION test.testVoidfun(dt date[])"
		"  RETURNS void AS "
		"$BODY$ "
		"DECLARE "
		"BEGIN "
		"  RAISE NOTICE 'test.testVoidfun arguments: %',dt; "
		"  RETURN; "
		"END "
		"$BODY$ "
		"  LANGUAGE 'plpgsql' VOLATILE "
		"  COST 100; ";
	dbExec->execSQL(sStmtCmd);
	try
	{
		pass = true;

		PG_date datearr(2, 1,1);

		PGdate datevar = {0, 1476,  9,  1, 2260432, 274, 0};
		datearr[0][0] = datevar;

		dbExec->addParam(PG_DATE_ARR, datearr);
		dbExec->execSQL("SELECT * FROM test.testVoidfun($1::date[])", 0);
	}
	catch(Smartdb::smartdb_exception& e)
	{
		std::cout << e.base().what() << std::endl;
		pass = false;
	}
	BOOST_CHECK_EQUAL(pass, true);
	sStmtCmd = "DROP FUNCTION IF EXISTS test.testVoidfun(dt date[])";
	dbExec->execSQL(sStmtCmd);
}

BOOST_AUTO_TEST_CASE( AutoRecconectTest )
{
	bool pass;
	conn_->setAutoReconnect(false);
	conn_->closeConnection();
	try
	{
		pass = false;
		dbExec->execSQL("SELECT 1");
	}
	catch(Smartdb::broken_connection)
	{
		pass = true;
	}
	BOOST_CHECK_EQUAL(pass, true);

	conn_->setAutoReconnect(true);
	//conn_->Connect();
	try
	{
		pass = true;
		dbExec->execSQL("SELECT 1");
	}
	catch(Smartdb::broken_connection)
	{
		pass = false;
	}
	BOOST_CHECK_EQUAL(pass, true);
}

BOOST_AUTO_TEST_CASE( BuildinDataTypesTest )
{
	PGbool boolvar = true, outboolvar;
	PGint2 int2var = std::numeric_limits<PGint2>::max() - 1, outint2var;
	PGint4 int4var = std::numeric_limits<PGint4>::max() - 1, outint4var;
	PGint8 int8var = std::numeric_limits<PGint8>::max() - 1, outint8var;
	PGfloat4 float4var = std::numeric_limits<PGfloat4>::max() - 1, outfloat4var;
	PGfloat8 float8var = std::numeric_limits<PGfloat8>::max() - 1, outfloat8var;
	PGvarchar varcharvar = "test string", outvarcharvar;
	PGdate datevar = {0, 1476,  9,  1, 2260432, 274, 0}, outdatevar;
	PGtimestamp timestampvar = {1198425427, {0, 2007, 11, 23, 2454458, 356, 0}, {15, 57,  7, 881542, 0, -1, 0, {0}}}, outtimestampvar;
	PGinet inetvar, outinetvar;
	PGmacaddr macaddrvar = {1,2,3,225,226,227}, outmacaddrvar;

	memset(&inetvar, 0, sizeof(PGinet));

	inetvar.mask = 32;
	inetvar.sa_buf_len = 16;
	inetvar.is_cidr = 0;
	inetvar.sa_buf[0] = (char)0x02;
	inetvar.sa_buf[4] = (char)0xc0;
	inetvar.sa_buf[5] = (char)0x06;
	inetvar.sa_buf[6] = (char)0x07;
	inetvar.sa_buf[7] = (char)0x08;

	new_buildin_table b;

	b.push_back(boost::shared_ptr<new_buildin_table::row_type>(new new_buildin_table::row_type));//reserved one row

	b.back().set_colbool(boolvar);
	b.back().set_colint2(int2var);
	b.back().set_colint4(int4var);
	b.back().set_colint8(int8var);
	b.back().set_colfloat4(float4var);
	b.back().set_colfloat8(float8var);

	b.back().set_colvarchar(varcharvar);
	b.back().set_coldate(datevar);
	b.back().set_coltimestamp(timestampvar);
	b.back().set_colinet(inetvar);
	b.back().set_colmacaddr(macaddrvar);

	std::string droptable = "DROP TABLE IF EXISTS test.buildintypes";
	dbExec->execSQL(droptable);

	std::string createtable = "CREATE TABLE test.buildintypes("
		"colbool bool NOT NULL,"
		"colint2 int2 NOT NULL,"
		"colint4 int4 NOT NULL,"
		"colint8 int8 NOT NULL,"
		"colfloat4 float4 NOT NULL,"
		"colfloat8 float8 NOT NULL,"
		"colvarchar varchar NOT NULL,"
		"coldate date NOT NULL,"
		"coltimestamp timestamp NOT NULL,"
		"colinet inet NOT NULL,"
		"colmacaddr macaddr NOT NULL,"
		"colmacaddr1 macaddr)";

	//"colvarchar varchar NOT NULL,"
	dbExec->execSQL(createtable);

	dbExec->execSQLSet(b, "INSERT INTO test.buildintypes VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12)");

	new_buildin_table outb;
	dbExec->execSQLGet(outb, "SELECT * FROM test.buildintypes");

    new_buildin_table::iterator it = outb.begin();
    for(; it != outb.end(); ++it)
    {
        outboolvar = (*it)->get_colbool();
        std::cout << outboolvar << std::endl;
    }
	/*BOOST_CHECK_EQUAL(outb.pOwnData.size(), 1);
	if (outb.pOwnData.size() != 1)
	{
	return;
	}*/

	outboolvar = outb.back().get_colbool();
	outint2var = outb.back().get_colint2();
	outint4var = outb.back().get_colint4();
	outint8var = outb.back().get_colint8();
	outfloat4var = outb.back().get_colfloat4();
	outfloat8var = outb.back().get_colfloat8();

	outvarcharvar = outb.back().get_colvarchar();
	outdatevar = outb.back().get_coldate();
	outtimestampvar = outb.back().get_coltimestamp();
	outinetvar = outb.back().get_colinet();
	outmacaddrvar = outb.back().get_colmacaddr();

	BOOST_CHECK_EQUAL(outboolvar, boolvar);
	BOOST_CHECK_EQUAL(outint2var, int2var);
	BOOST_CHECK_EQUAL(outint4var, int4var);
	BOOST_CHECK_EQUAL(outint8var, int8var);
	BOOST_CHECK_EQUAL(outfloat4var, float4var);
	BOOST_CHECK_EQUAL(outfloat8var, float8var);

	BOOST_CHECK(!outvarcharvar.compare(varcharvar));

	BOOST_CHECK(!memcmp(&outtimestampvar, &timestampvar, sizeof(PGtimestamp)));
	BOOST_CHECK(!memcmp(&outinetvar, &inetvar, sizeof(PGinet)));
	BOOST_CHECK(!memcmp(&outmacaddrvar, &macaddrvar, sizeof(PGmacaddr)));

	dbExec->execSQL(droptable);
}

BOOST_AUTO_TEST_CASE( BuildinTypesParamsTest )
{
	PGbool boolvar = true, outboolvar;
	PGint2 int2var = std::numeric_limits<PGint2>::max() - 1, outint2var;
	PGint4 int4var = std::numeric_limits<PGint4>::max() - 1, outint4var;
	PGint8 int8var = std::numeric_limits<PGint8>::max() - 1, outint8var;
	PGfloat4 float4var = std::numeric_limits<PGfloat4>::max() - 1, outfloat4var;
	PGfloat8 float8var = std::numeric_limits<PGfloat8>::max() - 1, outfloat8var;
	PGvarchar varcharvar = "test string", outvarcharvar;
	PGdate datevar = {0, 1476,  9,  1, 2260432, 274, 0}, outdatevar;
	PGtimestamp timestampvar = {1198425427, {0, 2007, 11, 23, 2454458, 356, 0}, {15, 57,  7, 881542, 0, -1, 0, {0}}}, outtimestampvar;
	PGinet inetvar, outinetvar;
	PGmacaddr macaddrvar = {1,2,3,225,226,227}, outmacaddrvar;

	memset(&inetvar, 0, sizeof(PGinet));

	inetvar.mask = 32;
	inetvar.sa_buf_len = 16;
	inetvar.is_cidr = 0;
	inetvar.sa_buf[0] = (char)0x02;
	inetvar.sa_buf[4] = (char)0xc0;
	inetvar.sa_buf[5] = (char)0x06;
	inetvar.sa_buf[6] = (char)0x07;
	inetvar.sa_buf[7] = (char)0x08;

	std::string droptable = "DROP TABLE IF EXISTS test.buildintypes";
	dbExec->execSQL(droptable);

	std::string createtable = "CREATE TABLE test.buildintypes("
		"colbool bool NOT NULL,"
		"colint2 int2 NOT NULL,"
		"colint4 int4 NOT NULL,"
		"colint8 int8 NOT NULL,"
		"colfloat4 float4 NOT NULL,"
		"colfloat8 float8 NOT NULL,"
		"colvarchar varchar NOT NULL,"
		"coldate date NOT NULL,"
		"coltimestamp timestamp NOT NULL,"
		"colinet inet NOT NULL,"
		"colmacaddr macaddr NOT NULL)";

	//"colvarchar varchar NOT NULL,"
	dbExec->execSQL(createtable);

	dbExec->addParam(PG_BOOL ,boolvar);
	dbExec->addParam(PG_INT2 ,int2var);
	dbExec->addParam(PG_INT4 ,int4var);
	dbExec->addParam(PG_INT8 ,int8var);

	dbExec->addParam(PG_FLOAT4 ,float4var);
	dbExec->addParam(PG_FLOAT8 ,float8var);

	dbExec->addParam(PG_VARCHAR ,varcharvar);
	dbExec->addParam(PG_DATE ,datevar);
	dbExec->addParam(PG_TIMESTAMP ,timestampvar);
	dbExec->addParam(PG_INET ,inetvar);
	dbExec->addParam(PG_MAC_ADDR ,macaddrvar);

	dbExec->execSQL("INSERT INTO test.buildintypes VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11)");

	new_buildin_table outb;
	dbExec->execSQLGet(outb, "SELECT * FROM test.buildintypes");

	/*BOOST_CHECK_EQUAL(outb.pOwnData.size(), 1);
	if (outb.pOwnData.size() != 1)
	{
	return;
	}*/

	outboolvar = outb.back().get_colbool();
	outint2var = outb.back().get_colint2();
	outint4var = outb.back().get_colint4();
	outint8var = outb.back().get_colint8();
	outfloat4var = outb.back().get_colfloat4();
	outfloat8var = outb.back().get_colfloat8();

	outvarcharvar = outb.back().get_colvarchar();
	outdatevar = outb.back().get_coldate();
	outtimestampvar = outb.back().get_coltimestamp();
	outinetvar = outb.back().get_colinet();
	outmacaddrvar = outb.back().get_colmacaddr();

	BOOST_CHECK_EQUAL(outboolvar, boolvar);
	BOOST_CHECK_EQUAL(outint2var, int2var);
	BOOST_CHECK_EQUAL(outint4var, int4var);
	BOOST_CHECK_EQUAL(outint8var, int8var);
	BOOST_CHECK_EQUAL(outfloat4var, float4var);
	BOOST_CHECK_EQUAL(outfloat8var, float8var);

	BOOST_CHECK(!outvarcharvar.compare(varcharvar));

	BOOST_CHECK(!memcmp(&outtimestampvar, &timestampvar, sizeof(PGtimestamp)));
	BOOST_CHECK(!memcmp(&outinetvar, &inetvar, sizeof(PGinet)));
	BOOST_CHECK(!memcmp(&outmacaddrvar, &macaddrvar, sizeof(PGmacaddr)));

	dbExec->execSQL(droptable);
}
BOOST_AUTO_TEST_CASE( ArrayTypesTest )
{
	PGbool boolvar = true, outboolvar;
	PGint2 int2var = std::numeric_limits<PGint2>::max() - 1, outint2var;
	PGint4 int4var = std::numeric_limits<PGint4>::max() - 1, outint4var;
	PGint8 int8var = std::numeric_limits<PGint8>::max() - 1, outint8var;
	PGfloat4 float4var = std::numeric_limits<PGfloat4>::max() - 1, outfloat4var;
	PGfloat8 float8var = std::numeric_limits<PGfloat8>::max() - 1, outfloat8var;
	PGvarchar varcharvar = "test string", outvarcharvar;
	PGdate datevar = {0, 1476,  9,  1, 2260432, 274, 0}, outdatevar;
	PGtimestamp timestampvar = {1198425427, {0, 2007, 11, 23, 2454458, 356, 0}, {15, 57,  7, 881542, 0, -1, 0, {0}}}, outtimestampvar;
	PGinet inetvar, outinetvar;
	PGmacaddr macaddrvar = {1,2,3,225,226,227}, outmacaddrvar;

	memset(&inetvar, 0, sizeof(PGinet));

	inetvar.mask = 32;
	inetvar.sa_buf_len = 16;
	inetvar.is_cidr = 0;
	inetvar.sa_buf[0] = (char)0x02;
	inetvar.sa_buf[4] = (char)0xc0;
	inetvar.sa_buf[5] = (char)0x06;
	inetvar.sa_buf[6] = (char)0x07;
	inetvar.sa_buf[7] = (char)0x08;

	PG_bool boolarr(2,1,1), outboolarr;
	PG_int2 int2arr(2,1,1), outint2arr;
	PG_int4 int4arr(2,1,1), outint4arr;
	PG_int8 int8arr(2,1,1), outint8arr;
	PG_float4 float4arr(2,1,1), outfloat4arr;
	PG_float8 float8arr(2,1,1), outfloat8arr;
	PG_varchar varchararr(2,1,1), outvarchararr;
	PG_date datearr(2,1,1), outdatearr;
	PG_timestamp timestamparr(2,1,1), outtimestamparr;
	PG_inet inetarr(2,1,1), outinetarr;
	PG_macaddr macaddrarr(2,1,1), outmacaddrarr;

	boolarr[0][0] = boolvar;
	int2arr[0][0] = int2var;
	int4arr[0][0] = int4var;
	int8arr[0][0] = int8var;
	float4arr[0][0] = float4var;
	float8arr[0][0] = float8var;
	varchararr[0][0] = varcharvar;
	timestamparr[0][0] = timestampvar;
	datearr[0][0] = datevar;
	inetarr[0][0] = inetvar;
	macaddrarr[0][0] = macaddrvar;

	new_arraytypes_table a;

	a.push_back(boost::shared_ptr<new_arraytypes_table::row_type>(new new_arraytypes_table::row_type));//reserved one row

	a.back().set_colbool(boolarr);
	a.back().set_colint2(int2arr);
	a.back().set_colint4(int4arr);
	a.back().set_colint8(int8arr);

	a.back().set_colfloat4(float4arr);
	a.back().set_colfloat8(float8arr);

	a.back().set_colvarchar(varchararr);
	a.back().set_coldate(datearr);
	a.back().set_coltimestamp(timestamparr);
	a.back().set_colinet(inetarr);
	a.back().set_colmacaddr(macaddrarr);

	std::string droptable = "DROP TABLE IF EXISTS test.arraytypes";
	dbExec->execSQL(droptable);

	std::string createtable = "CREATE TABLE test.arraytypes("
		"colbool bool[],"
		"colint2 int2[],"
		"colint4 int4[],"
		"colint8 int8[],"
		"colfloat4 float4[],"
		"colfloat8 float8[],"
		"colvarchar varchar[],"
		"coldate date[],"
		"coltimestamp timestamp[],"
		"colinet inet[],"
		"colmacaddr macaddr[],"
		"nullarr int2[])";
	//"colvarchar varchar[],"

	dbExec->execSQL(createtable);
	//dbExec->execSQLSet(a, "INSERT INTO test.arraytypes VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, NULL)");
    dbExec->execSQLSet(a, "INSERT INTO test.arraytypes VALUES($1, $2, '{}', $4, $5, $6, $7, $8, $9, $10, $11, NULL)");

	new_arraytypes_table outa;
	dbExec->execSQLGet(outa, "SELECT * FROM test.arraytypes");

	outboolarr = outa.back().get_colbool();
	outint2arr = outa.back().get_colint2();
	outint4arr = outa.back().get_colint4();
	outint8arr = outa.back().get_colint8();

	outfloat4arr = outa.back().get_colfloat4();
	outfloat8arr = outa.back().get_colfloat8();

	outvarchararr = outa.back().get_colvarchar();
	outdatearr = outa.back().get_coldate();
	outtimestamparr = outa.back().get_coltimestamp();
	outinetarr = outa.back().get_colinet();
	outmacaddrarr = outa.back().get_colmacaddr();

	BOOST_CHECK(outint2arr.get_num_dims() == 2 );
	BOOST_CHECK(outint4arr.get_num_dims() == 0 );
	BOOST_CHECK(outint8arr.get_num_dims() == 2 );

	BOOST_CHECK(outfloat4arr.get_num_dims() == 2 );
	BOOST_CHECK(outfloat8arr.get_num_dims() == 2 );

	BOOST_CHECK(outdatearr.get_num_dims() == 2 );
	BOOST_CHECK(outtimestamparr.get_num_dims() == 2 );
	BOOST_CHECK(outinetarr.get_num_dims() == 2 );
	BOOST_CHECK(outmacaddrarr.get_num_dims() == 2 );

	outboolvar = outboolarr[0][0];
	outint2var = outint2arr[0][0];
	outint4var = outint4arr[0].isNull() ? 0 : outint4arr[0];
	outint8var = outint8arr[0][0];
	outfloat4var = outfloat4arr[0][0];
	outfloat8var = outfloat8arr[0][0];
	outvarcharvar = outvarchararr[0][0];
	outdatevar = outdatearr[0][0];
	outtimestampvar = outtimestamparr[0][0];
	outinetvar = outinetarr[0][0];
	outmacaddrvar = outmacaddrarr[0][0];

	BOOST_CHECK(outboolvar == boolvar);
	BOOST_CHECK(outint2var == int2var);
	BOOST_CHECK(outint4var == 0);
	BOOST_CHECK(outint8var == int8var);

	BOOST_CHECK(outfloat4var == float4var);
	BOOST_CHECK(outfloat8var == float8var);

	BOOST_CHECK(!outvarcharvar.compare(varcharvar));
	BOOST_CHECK(!memcmp(&outdatevar, &datevar, sizeof(PGdate)));
	BOOST_CHECK(!memcmp(&outtimestampvar, &timestampvar, sizeof(PGtimestamp)));
	BOOST_CHECK(!memcmp(&outinetvar, &inetvar, sizeof(PGinet)));
	BOOST_CHECK(!memcmp(&outmacaddrvar, &macaddrvar, sizeof(PGmacaddr)));

	dbExec->execSQL(droptable);
}

BOOST_AUTO_TEST_CASE( ArrayParamTypesTest )
{
	PGbool boolvar = true, outboolvar;
	PGint2 int2var = std::numeric_limits<PGint2>::max() - 1, outint2var;
	PGint4 int4var = std::numeric_limits<PGint4>::max() - 1, outint4var;
	PGint8 int8var = std::numeric_limits<PGint8>::max() - 1, outint8var;
	PGfloat4 float4var = std::numeric_limits<PGfloat4>::max() - 1, outfloat4var;
	PGfloat8 float8var = std::numeric_limits<PGfloat8>::max() - 1, outfloat8var;
	PGvarchar varcharvar = "test string", outvarcharvar;
	PGdate datevar = {0, 1476,  9,  1, 2260432, 274, 0}, outdatevar;
	PGtimestamp timestampvar = {1198425427, {0, 2007, 11, 23, 2454458, 356, 0}, {15, 57,  7, 881542, 0, -1, 0, {0}}}, outtimestampvar;
	PGinet inetvar, outinetvar;
	PGmacaddr macaddrvar = {1,2,3,225,226,227}, outmacaddrvar;

	memset(&inetvar, 0, sizeof(PGinet));

	inetvar.mask = 32;
	inetvar.sa_buf_len = 16;
	inetvar.is_cidr = 0;
	inetvar.sa_buf[0] = (char)0x02;
	inetvar.sa_buf[4] = (char)0xc0;
	inetvar.sa_buf[5] = (char)0x06;
	inetvar.sa_buf[6] = (char)0x07;
	inetvar.sa_buf[7] = (char)0x08;

	PG_bool boolarr(2,1,1), outboolarr;
	PG_int2 int2arr(2,1,1), outint2arr;
	PG_int4 int4arr(2,1,1), outint4arr;
	PG_int8 int8arr(2,1,1), outint8arr;
	PG_float4 float4arr(2,1,1), outfloat4arr;
	PG_float8 float8arr(2,1,1), outfloat8arr;
	PG_varchar varchararr(2,1,1), outvarchararr;
	PG_date datearr(2,1,1), outdatearr;
	PG_timestamp timestamparr(2,1,1), outtimestamparr;
	PG_inet inetarr(2,1,1), outinetarr;
	PG_macaddr macaddrarr(2,1,1), outmacaddrarr;

	boolarr[0][0] = boolvar;
	int2arr[0][0] = int2var;
	int4arr[0][0] = int4var;
	int8arr[0][0] = int8var;
	float4arr[0][0] = float4var;
	float8arr[0][0] = float8var;
	varchararr[0][0] = varcharvar;
	timestamparr[0][0] = timestampvar;
	datearr[0][0] = datevar;
	inetarr[0][0] = inetvar;
	macaddrarr[0][0] = macaddrvar;

	std::string droptable = "DROP TABLE IF EXISTS test.arraytypes";
	dbExec->execSQL(droptable);

	std::string createtable = "CREATE TABLE test.arraytypes("
		"colbool bool[],"
		"colint2 int2[],"
		"colint4 int4[],"
		"colint8 int8[],"
		"colfloat4 float4[],"
		"colfloat8 float8[],"
		"colvarchar varchar[],"
		"coldate date[],"
		"coltimestamp timestamp[],"
		"colinet inet[],"
		"colmacaddr macaddr[],"
		"nullarr int2[])";
	//"colvarchar varchar[],"

	dbExec->execSQL(createtable);

	dbExec->addParam(PG_BOOL_ARR, boolarr);
	dbExec->addParam(PG_INT2_ARR, int2arr);
	dbExec->addParam(PG_INT4_ARR, int4arr);
	dbExec->addParam(PG_INT8_ARR, int8arr);

	dbExec->addParam(PG_FLOAT4_ARR, float4arr);
	dbExec->addParam(PG_FLOAT8_ARR, float8arr);


	dbExec->addParam(PG_VARCHAR_ARR, varchararr);
	dbExec->addParam(PG_DATE_ARR, datearr);
	dbExec->addParam(PG_TIMESTAMP_ARR, timestamparr);

	dbExec->addParam(PG_INET_ARR, inetarr);
	dbExec->addParam(PG_MAC_ADDR_ARR, macaddrarr);

	dbExec->execSQL("INSERT INTO test.arraytypes VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11)");

	new_arraytypes_table outa;
	dbExec->execSQLGet(outa, "SELECT * FROM test.arraytypes");

	outboolarr = outa.back().get_colbool();
	outint2arr = outa.back().get_colint2();
	outint4arr = outa.back().get_colint4();
	outint8arr = outa.back().get_colint8();

	outfloat4arr = outa.back().get_colfloat4();
	outfloat8arr = outa.back().get_colfloat8();

	outvarchararr = outa.back().get_colvarchar();
	outdatearr = outa.back().get_coldate();
	outtimestamparr = outa.back().get_coltimestamp();
	outinetarr = outa.back().get_colinet();
	outmacaddrarr = outa.back().get_colmacaddr();

	BOOST_CHECK(outint2arr.get_num_dims() == 2 );
	BOOST_CHECK(outint4arr.get_num_dims() == 2 );
	BOOST_CHECK(outint8arr.get_num_dims() == 2 );

	BOOST_CHECK(outfloat4arr.get_num_dims() == 2 );
	BOOST_CHECK(outfloat8arr.get_num_dims() == 2 );

	BOOST_CHECK(outdatearr.get_num_dims() == 2 );
	BOOST_CHECK(outtimestamparr.get_num_dims() == 2 );
	BOOST_CHECK(outinetarr.get_num_dims() == 2 );
	BOOST_CHECK(outmacaddrarr.get_num_dims() == 2 );

	outboolvar = outboolarr[0][0];
	outint2var = outint2arr[0][0];
	outint4var = outint4arr[0][0];
	outint8var = outint8arr[0][0];
	outfloat4var = outfloat4arr[0][0];
	outfloat8var = outfloat8arr[0][0];
	outvarcharvar = outvarchararr[0][0];
	outdatevar = outdatearr[0][0];
	outtimestampvar = outtimestamparr[0][0];
	outinetvar = outinetarr[0][0];
	outmacaddrvar = outmacaddrarr[0][0];

	BOOST_CHECK(outboolvar == boolvar);
	BOOST_CHECK(outint2var == int2var);
	BOOST_CHECK(outint4var == int4var);
	BOOST_CHECK(outint8var == int8var);

	BOOST_CHECK(outfloat4var == float4var);
	BOOST_CHECK(outfloat8var == float8var);

	BOOST_CHECK(!outvarcharvar.compare(varcharvar));
	BOOST_CHECK(!memcmp(&outdatevar, &datevar, sizeof(PGdate)));
	BOOST_CHECK(!memcmp(&outtimestampvar, &timestampvar, sizeof(PGtimestamp)));
	BOOST_CHECK(!memcmp(&outinetvar, &inetvar, sizeof(PGinet)));
	BOOST_CHECK(!memcmp(&outmacaddrvar, &macaddrvar, sizeof(PGmacaddr)));

	dbExec->execSQL(droptable);
}
BOOST_FIXTURE_TEST_CASE( linkTest , InstTableSetup)
{
	std::string dropTableCmd = "DROP TABLE IF EXISTS test.asset";
	dbExec->execSQL(dropTableCmd);

	std::string createTableCmd = "CREATE TABLE test.asset"
		"("
		"inst_id integer NOT NULL,"
		"widget_id integer NOT NULL"
		");";
	dbExec->execSQL(createTableCmd);
	dbExec->execSQL("INSERT INTO test.asset VALUES (1, 1)");
	dbExec->execSQL("INSERT INTO test.asset VALUES (2, 1)");

	asset_tradable_combination_table toLink;

	dbExec->execSQLGet(testTbl, "SELECT * from test.inst");
	dbExec->execSQLGet(toLink, "SELECT * from test.asset");

	typedef external_link_cnt<inst_table::row_type, asset_tradable_combination_table::row_type> lr;
	lr res1;
	dbExec->linkRows<0,0>(testTbl, toLink, res1);

	typedef external_link_cnt<asset_tradable_combination_table::row_type, inst_table::row_type> lr1;
	lr1 res2;
	dbExec->linkRows<1,0>(toLink, testTbl, res2);

	unsigned sum = 0;
	lr::iterator i = res1.begin();
	for(; i != res1.end(); ++i)
	{
		sum += (*i).second.size();
	}

	BOOST_CHECK_EQUAL( sum, 3);

	unsigned sum1 = 0;
	lr1::iterator i1 = res2.begin();
	for(; i1 != res2.end(); ++i1)
	{
		sum1 += (*i1).second.size();
	}

	BOOST_CHECK_EQUAL( sum1, 2);
}
/*
BOOST_AUTO_TEST_CASE( vectorTest )
{
Net_test::net_test_ testNetTbl;

char testIpVal = 10;
int testMacVal = 15;
std::string dropOldTableInstanceCmd = "DROP TABLE IF EXISTS test.net_test";

dbExec->execSQL(dropOldTableInstanceCmd);

std::string createTableCmd = "CREATE TABLE test.net_test("
"inets inet[] NOT NULL,"
"mac_addrs macaddr[] NOT NULL"
")";		
dbExec->execSQL(createTableCmd.c_str());

std::string insertTableCmd="INSERT INTO test.net_test("
"inets, mac_addrs)"
"VALUES ('{{""192.6.7.8""}}', '{{""12:34:56:78:91:ab""}}');";

dbExec->execSQL(insertTableCmd);

dbExec->execSQLGet(testNetTbl, "SELECT * FROM test.net_test");

std::size_t vecSize = testNetTbl.pOwnData.size();	

BOOST_CHECK_EQUAL( vecSize, 1);

if (vecSize > 0)
{
PG_inet_matrix m;

testNetTbl.resetReflector(*(testNetTbl.pOwnData.begin()));
testNetTbl.pReflector->pOwnData->P_get_inets(&m);

//m(0,0).mask = maskVal;

BOOST_CHECK_EQUAL( m.size1(), 1);
BOOST_CHECK_EQUAL( m.size2(), 1);

if ( m.size1() == 1 && m.size2() == 1)
{
char c = m(0,0).sa_buf[4];
BOOST_CHECK_EQUAL( m(0,0).sa_buf[4], (char)0xC0);
BOOST_CHECK_EQUAL( m(0,0).sa_buf[5], (char)0x6);
BOOST_CHECK_EQUAL( m(0,0).sa_buf[6], (char)0x7);
BOOST_CHECK_EQUAL( m(0,0).sa_buf[7], (char)0x8);
m(0,0).sa_buf[4] = testIpVal;
}

testNetTbl.pReflector->pOwnData->P_set_inets(&m);

PG_macaddr_matrix maddr;
testNetTbl.pReflector->pOwnData->P_get_mac_addrs(&maddr);

BOOST_CHECK_EQUAL( maddr.size1(), 1);
BOOST_CHECK_EQUAL( maddr.size2(), 1);

if ( maddr.size1() == 1 && maddr.size2() == 1)
{
BOOST_CHECK_EQUAL( maddr(0,0).a, (int)0x12);
BOOST_CHECK_EQUAL( maddr(0,0).b, (int)0x34);
BOOST_CHECK_EQUAL( maddr(0,0).c, (int)0x56);
BOOST_CHECK_EQUAL( maddr(0,0).d, (int)0x78);
BOOST_CHECK_EQUAL( maddr(0,0).e, (int)0x91);
BOOST_CHECK_EQUAL( maddr(0,0).f, (int)0xab);
maddr(0,0).a = testMacVal;
}
testNetTbl.pReflector->pOwnData->P_set_mac_addrs(&maddr);
dbExec->execSQLSet(testNetTbl, "INSERT INTO test.net_test VALUES($1,$2)");
}
testNetTbl.pOwnData.clear();

dbExec->execSQLGet(testNetTbl, "SELECT * FROM test.net_test");

BOOST_CHECK_EQUAL( testNetTbl.pOwnData.size(), 2);

std::size_t newVecSize = testNetTbl.pOwnData.size();	
if (newVecSize > 1)
{
PG_inet_matrix m;
testNetTbl.resetReflector(*(testNetTbl.pOwnData.begin() + 1));
testNetTbl.pReflector->pOwnData->P_get_inets(&m);

BOOST_CHECK_EQUAL( m.size1() , 1);
BOOST_CHECK_EQUAL( m.size2() , 1);

if (m.size1() == 1 && m.size1() == 1 )
{
BOOST_CHECK_EQUAL(m(0,0).sa_buf[4], testIpVal );
}
PG_macaddr_matrix maddr;
testNetTbl.pReflector->pOwnData->P_get_mac_addrs(&maddr);

BOOST_CHECK_EQUAL( maddr.size1() , 1);
BOOST_CHECK_EQUAL( maddr.size2() , 1);

if (maddr.size1() == 1 && maddr.size1() == 1 )
{
BOOST_CHECK_EQUAL(maddr(0,0).a, (int)testMacVal);
}
}
dbExec->execSQL(dropOldTableInstanceCmd);
testNetTbl.pOwnData.clear();
}
*/
void myNote(const char* m)
{
	std::cout << "Custom Notice: " << m << std::endl;
}
BOOST_FIXTURE_TEST_CASE( PGint8ArrParamInProcedure, InstTableSetup )
{
	struct A
	{
		static void myNoticeReceiver(const char *msg)
		{
			std::cout << "A message from myNoticeReceiver: " << msg << std::endl;
		}
		static void myNoticeReceiver2(const char *msg)
		{
			std::cout << "A message from myNoticeReceiver2: " << msg << std::endl;
		}
	};
	//dbExec->setCustomNoticeReceiver(&A::myNoticeReceiver);
	dbExec->setCustomNoticeReceiver(myNote);
	std::string stmt = "CREATE OR REPLACE FUNCTION test.get_(cur_inst_vec_id integer[]) "
		"  RETURNS SETOF record AS "
		"$BODY$ "
		"DECLARE "
		" str varchar; "
		"BEGIN "
		"  str := array_to_string(cur_inst_vec_id, ' | ');	 "
		"  RAISE NOTICE 'get_ arguments: %', str; "
		"  IF (SELECT count(*) FROM (SELECT elem FROM common.util_explode_array(cur_inst_vec_id) AS elem "
		"	WHERE elem NOT IN (SELECT inst_vector_id FROM common_inst.index_element_inst_vector)) AS ie_table)>0 THEN "
		"	RAISE EXCEPTION 'Not all inst_vector_id FROM % are registered in database',cur_inst_vec_id; "
		"  END IF; "
		"  RETURN QUERY SELECT inst_vector_id,ARRAY(SELECT element_id FROM common_inst.index_element_inst_vector "
		"  WHERE inst_vector_id=alias_index_element_inst_vector.inst_vector_id "
		"  ORDER BY element_position) FROM common_inst.index_element_inst_vector alias_index_element_inst_vector "
		"  WHERE inst_vector_id=ANY($1) GROUP BY inst_vector_id; "
		"END "
		"$BODY$ "
		"  LANGUAGE 'plpgsql' VOLATILE "
		"  COST 100 "
		"  ROWS 1000; ";

	dbExec->execSQL(stmt);	

	stmt = dbExec->prepareAutoNameSQL("SELECT * FROM test.get_($1) AS (inst_vector_id integer,inst_id_list bigint[]);");

	PG_int4 mat(2, 4, 1);

	mat[0][0] = 2;
	mat[1][0] = 3;
	mat[2][0] = 4;
	mat[3][0] = 5;

	dbExec->addParam(PG_INT4_ARR, mat);

	instruments_in_inst_vec_table instruments_in_instid_vec;
	try
	{
		dbExec->execPreparedSQLGet(instruments_in_instid_vec, stmt);
	}
	catch(Smartdb::smartdb_exception& ex)
	{
		ex.base().what();
		dbExec->deallocPreparedSQL(stmt);
		dbExec->clearNoticeReceiver();
	}

	dbExec2->execSQL("DROP FUNCTION IF EXISTS test.get_(cur_inst_vec_id integer[])");
	dbExec2->setCustomNoticeReceiver(&A::myNoticeReceiver2);

	dbExec2->execSQL("DROP FUNCTION IF EXISTS test.get_AAAA(cur_inst_vec_id integer[])");
	dbExec2->clearNoticeReceiver();

	boost::shared_ptr<ConnectionManager> conn_2;
	conn_2.reset(new ConnectionManager(connectionInfo, g_logger.get()));
	dbExec3.reset(new CDBExecutor(conn_2.get(), g_logger.get()));

	dbExec3->execSQL("DROP FUNCTION IF EXISTS test.get_BBBB(cur_inst_vec_id integer[])");
	dbExec3->setCustomNoticeReceiver(&A::myNoticeReceiver2);
	dbExec3->execSQL("DROP FUNCTION IF EXISTS test.get_BBBB(cur_inst_vec_id integer[])");
	dbExec3->clearNoticeReceiver();
	dbExec3->execSQL("DROP FUNCTION IF EXISTS test.get_BBBB(cur_inst_vec_id integer[])");
	//////////////////////////////////////
	boost::shared_ptr<ConnectionManager> conn1_;
	boost::shared_ptr<ConnectionManager> conn2_;

	boost::shared_ptr<CDBExecutor> dbExec1_;
	boost::shared_ptr<CDBExecutor> dbExec2_;

	conn1_.reset(new ConnectionManager(connectionInfo, g_logger.get()));
	conn2_.reset(new ConnectionManager(connectionInfo, g_logger.get()));

	dbExec1_.reset( new CDBExecutor(conn1_.get(), g_logger.get()));
	dbExec2_.reset( new CDBExecutor(conn2_.get(), g_logger.get()));

	std::string stmtName1 = dbExec1_->prepareAutoNameSQL("SELECT 1");

	dbExec1_->execPreparedSQL(stmtName1);

	bool pass = false;
	std::string stmtName2;
	try
	{
		stmtName2 = dbExec1_->prepareAutoNameSQL("SELECT 1 from undefined.dummy");
	}
	catch(Smartdb::sql_error& ex)
	{
		ex.what();
		pass = true;
	}
	BOOST_CHECK(pass);

	try
	{
		pass = false;
		dbExec2_->execPreparedSQL(stmtName2);
	}
	catch(Smartdb::usage_error& ex)
	{
		ex.what();
		pass = true;
	}

	BOOST_CHECK(pass);

	dbExec1_->deallocPreparedSQL(stmtName1);
	try
	{
		pass = false;
		dbExec2_->deallocPreparedSQL(stmtName2);
	}
	catch(Smartdb::usage_error& ex)
	{
		ex.what();
		pass = true;
	}
	BOOST_CHECK(pass);
};

BOOST_AUTO_TEST_CASE( ArrayParamInProcedure )
{
	std::string createFunctionCmd = "CREATE OR REPLACE FUNCTION test.testfun(int4[], macaddr[])"
		"RETURNS integer AS $$ "
		"BEGIN "
		"CREATE TABLE test.a(s varchar, b varchar);"
		"INSERT INTO test.a VALUES(array_to_string($1, '|'), array_to_string($2, '~'));"
		"RETURN 1;"
		"END;"
		"$$ LANGUAGE plpgsql;";

	PG_int4 m(2, 10, 10);

	for(int i = 0; i < 10; ++i)
		for(int j = 0; j < 10; ++j)
			m[i][j] = i + j;

	PG_macaddr maddr(2, 2, 2);

	PGmacaddr mm1 = {12,34,56,78,91,0};
	maddr[0][0] = mm1;

	PGmacaddr mm2 = {11,32,16,18,11,40};
	maddr[0][1] = mm2;

	dbExec->execSQL("DROP TABLE IF EXISTS test.a");
	dbExec->execSQL(createFunctionCmd);

	dbExec->addParam(PG_INT4_ARR, m);
	dbExec->addParam(PG_MAC_ADDR_ARR, maddr);
	try
	{
		dbExec->execSQL("SELECT test.testfun($1, $2)");
	}
	catch(Smartdb::smartdb_exception& e)
	{
		std::cout << e.base().what() << std::endl;
	}
	dbExec->execSQL("DROP FUNCTION IF EXISTS test.testfun(int4[], macaddr[])");
	dbExec->execSQL("SELECT * from test.a");
	dbExec->execSQL("DROP TABLE IF EXISTS test.a");

	//	BOOST_CHECK_EQUAL(testTbl.pOwnData.size(), 1);
}

BOOST_FIXTURE_TEST_CASE( SelectTest , InstTableSetup)
{
	dbExec->execSQLGet(testTbl, "SELECT * FROM test.inst");
	BOOST_CHECK_EQUAL( testTbl.size(), 3);
}

BOOST_FIXTURE_TEST_CASE( SelectTestSelectPrepared, InstTableSetup )
{
	std::string stmtName = dbExec->prepareAutoNameSQL("SELECT * FROM test.inst where inst_id = $1");
	dbExec->addParam(PG_INT4, 1);
	dbExec->execPreparedSQLGet(testTbl, stmtName);
	BOOST_CHECK_EQUAL( testTbl.size(), 1);

	testTbl.clear();

	dbExec->addParam(PG_INT4, 2);
	dbExec->execPreparedSQLGet(testTbl, stmtName);

	BOOST_CHECK_EQUAL( testTbl.size(), 2);

	dbExec->deallocPreparedSQL(stmtName);
}

BOOST_FIXTURE_TEST_CASE( InsertTest, InstTableSetup )
{
	//select 3 rows
	dbExec->prepareSQL("SELECT * FROM test.inst", "select");
	dbExec->execPreparedSQLGet(testTbl, "select");
	//insert selected rows into table
	dbExec->execSQLSet(testTbl, "INSERT INTO test.inst VALUES($1, $2, $3, $4, $5, $6, $7)");

	testTbl.clear();
	//now in table are 6 rows of data
	dbExec->execPreparedSQLGet(testTbl, "select");

	dbExec->deallocPreparedSQL("select");

	BOOST_CHECK_EQUAL( testTbl.size(), 6);
}

BOOST_FIXTURE_TEST_CASE( InsertTestPrepared, InstTableSetup )
{
	//select 3 rows
	dbExec->prepareSQL("SELECT * FROM test.inst", "select123");
	dbExec->execPreparedSQLGet(testTbl, "select123");
	//insert selected into table
	dbExec->prepareSQL("INSERT INTO test.inst VALUES($1, $2, $3, $4, $5, $6,$7)", "insert123");
	dbExec->execPreparedSQLSet(testTbl, "insert123");

	testTbl.clear();
	//now in table are 6 rows of data
	dbExec->execPreparedSQLGet(testTbl, "select123");

	dbExec->deallocPreparedSQL("insert123");
	dbExec->deallocPreparedSQL("select123");

	BOOST_CHECK_EQUAL( testTbl.size(), 6);
}

BOOST_FIXTURE_TEST_CASE( ProcedureCall, InstTableSetup )
{
	std::string createFunctionCmd = "CREATE OR REPLACE FUNCTION test.testfun(int, int, int, int, double precision, text, timestamp)"
		"RETURNS integer AS $$ "
		"BEGIN "
		"DELETE FROM test.inst WHERE inst_id = $1 + 1;"
		"RETURN 1;"
		"END;"
		"$$ LANGUAGE plpgsql;";

	dbExec->execSQL(createFunctionCmd);

	dbExec->execSQLGet(testTbl, "SELECT * FROM test.inst");

	//select 3 rows
	dbExec->prepareSQL("SELECT test.testfun($1,$2,$3,$4,$5,$6, $7);", "select");
	dbExec->execPreparedSQLSet(testTbl, "select");

	testTbl.clear();

	dbExec->execSQLGet(testTbl, "SELECT * FROM test.inst");

	dbExec->execSQL("DROP FUNCTION IF EXISTS test.testfun(int, int, int, int, double precision, text, timestamp)");

	dbExec->deallocPreparedSQL("select");

	BOOST_CHECK_EQUAL(testTbl.size(), 1);
}

BOOST_AUTO_TEST_CASE( NetTest )
{
	net_test_table testNetTbl;
	std::string dropOldTableInstanceCmd = "DROP TABLE IF EXISTS test.net_test";

	dbExec->execSQL(dropOldTableInstanceCmd);

	std::string createTableCmd = "CREATE TABLE test.net_test("
		"inets inet[] NOT NULL,"
		"mac_addrs macaddr[] NOT NULL"
		")";		
	dbExec->execSQL(createTableCmd.c_str());

	std::string insertTableCmd="INSERT INTO test.net_test("
		"inets, mac_addrs)"
		"VALUES ('{{""192.6.7.8""},{""192.6.7.9""}}', '{{""12:34:56:78:91:ab""}, {""22:34:56:78:90:ab""}}');";

	dbExec->execSQL(insertTableCmd);

	dbExec->execSQLGet(testNetTbl, "SELECT * FROM test.net_test");

	const int maskVal = 5, macVal = 9;

	std::size_t vecSize = testNetTbl.size();
	if (vecSize > 0)
	{
		PG_inet &m  = testNetTbl.back().get_inets();
		
		m[0][0]->mask = maskVal;

		testNetTbl.back().set_inets(m);

		PG_macaddr &maddr = testNetTbl.back().get_mac_addrs();
		maddr[0][0]->a = macVal;

		testNetTbl.back().set_mac_addrs(maddr);

		dbExec->execSQLSet(testNetTbl, "INSERT INTO test.net_test VALUES($1,$2)");
	}
	testNetTbl.clear();
	dbExec->execSQLGet(testNetTbl, "SELECT * FROM test.net_test");

	BOOST_CHECK_EQUAL( testNetTbl.size(), 2);

	std::size_t newVecSize = testNetTbl.size();	
	if (newVecSize > 1)
	{
		PG_inet &m  = testNetTbl.back().get_inets();
		BOOST_CHECK_EQUAL( ((PGinet&)m[0][0]).mask , maskVal);

		PG_macaddr &maddr  = testNetTbl.back().get_mac_addrs();
		BOOST_CHECK_EQUAL( ((PGmacaddr&)maddr[0][0]).a , macVal);
	}
	dbExec->execSQL(dropOldTableInstanceCmd);
}

BOOST_AUTO_TEST_CASE( PrepareAutoNameTest )
{
	inst_table testTbl;
	boost::shared_ptr<ConnectionManager> conn1_;
	boost::shared_ptr<ConnectionManager> conn2_;

	boost::shared_ptr<CDBExecutor> dbExec1;
	boost::shared_ptr<CDBExecutor> dbExec2;

	conn1_.reset(new ConnectionManager(connectionInfo, g_logger.get()));
	conn2_.reset(new ConnectionManager(connectionInfo, g_logger.get()));

	dbExec1.reset( new CDBExecutor(conn1_.get(), g_logger.get()));
	dbExec2.reset( new CDBExecutor(conn2_.get(), g_logger.get()));

	std::string stmtName1 = dbExec1->prepareAutoNameSQL("SELECT 1");
	std::string stmtName2 = dbExec1->prepareAutoNameSQL("SELECT 1");
	std::string stmtName3 = dbExec2->prepareAutoNameSQL("SELECT 1");

	dbExec1->deallocPreparedSQL(stmtName1);
	dbExec1->deallocPreparedSQL(stmtName2);
	dbExec2->deallocPreparedSQL(stmtName3);


	BOOST_CHECK_NE(stmtName1, stmtName2);
	BOOST_CHECK_NE(stmtName1, stmtName3);
	BOOST_CHECK_NE(stmtName2, stmtName3);
}

BOOST_FIXTURE_TEST_CASE( undefined_tableTest , InstTableSetup)
{
	inst_table testTbl;
	std::string errMsg;
	std::string place;
	bool pass = false;

	try
	{
		dbExec->prepareSQL("SELECT * FROM test.nst", "select");
		dbExec->execPreparedSQLGet(testTbl, "select");
	}
	catch (Smartdb::undefined_table &err)
	{
		errMsg = err.what();
		place = err.getErrPoint();

		pass = true;
	}

	BOOST_CHECK_EQUAL( pass, true);
	BOOST_CHECK_GT( errMsg.length(), 0);
	BOOST_CHECK_GT( place.length(), 0);
}

BOOST_AUTO_TEST_CASE( sql_errorTest )
{
	std::string errMsg;
	std::string place;
	bool pass = false;

	try
	{
		dbExec->execSQL("SELECTFROM test.nst");
	}
	catch (Smartdb::sql_error &err)
	{
		errMsg = err.what();
		place = err.getErrPoint();

		pass = true;
	}
	catch(Smartdb::smartdb_exception&)
	{
		pass = false;
	}

	BOOST_CHECK_EQUAL( pass, true);
	BOOST_CHECK_GT( errMsg.length(), 0);
	BOOST_CHECK_GT( place.length(), 0);
}

BOOST_AUTO_TEST_CASE( broken_connectionTest )
{
	InitLoger();
	conn_.reset(new ConnectionManager("invalid connection info", g_logger.get()));
	dbExec.reset( new CDBExecutor(conn_.get(), g_logger.get()));

	bool pass = false;
	try
	{
		conn_->getConnection();
	}
	catch (Smartdb::broken_connection &err)
	{
		err.what();
		pass = true;
	}
	BOOST_CHECK_EQUAL( pass, true);
}

BOOST_AUTO_TEST_CASE( SmartdbexceptionsTest )
{
	InitLoger();
	conn_.reset(new ConnectionManager("invalid connection info", g_logger.get()));
	dbExec.reset( new CDBExecutor(conn_.get(), g_logger.get()));

	bool pass = false;
	try
	{
		try
		{
			conn_->getConnection();
		}
		catch (Smartdb::broken_connection &err)
		{
			err.what();
			conn_.reset(new ConnectionManager(connectionInfo, g_logger.get()));
			dbExec.reset( new CDBExecutor(conn_.get(), g_logger.get()));
			dbExec->execSQL("SELECT 1");
			pass = true;
		}
	}
	catch(...)
	{
		pass = false;
	}
	BOOST_CHECK_EQUAL( pass, true);
}
BOOST_FIXTURE_TEST_CASE( TimestampTest , InstTableSetup)
{
	inst_table testTbl;
	dbExec->execSQLGet(testTbl, "SELECT * FROM test.inst");


	std::size_t newVecSize = testTbl.size();	
	if (newVecSize > 1)
	{
		PGtimestamp &t = testTbl.back().get_time();

		BOOST_CHECK_EQUAL( t.date.year, 2001);
		BOOST_CHECK_EQUAL( t.time.hour, 23);
	}
}

static void smartdmerrorHandler(const Smartdb::smartdb_exception *e)
{
}
BOOST_AUTO_TEST_CASE( SmartdbErrorHandlerTest )
{
	bool pass = false;
	try
	{
		dbExec->setErrorHandler((SmartDBErrHandlerFunc)&smartdmerrorHandler);
		dbExec->execSQL("sdfsdgsdfghdf");
		pass = true;
	}
	catch (Smartdb::smartdb_exception &err)
	{
		err.base().what();
		pass = false;
	}
	BOOST_CHECK_EQUAL( pass, true);

	bool pass2 = false;
	try
	{
		dbExec->setErrorHandler(NULL);
		dbExec->execSQL("sdfsdgsdfghdf");
	}
	catch (Smartdb::smartdb_exception &err)
	{
		err.base().what();
		pass2 = true;
	}
	BOOST_CHECK_EQUAL( pass2, true);
}

BOOST_AUTO_TEST_CASE( Sort_Test )
{
	std::string dropTableCmd = "DROP TABLE IF EXISTS test.asset";
	dbExec->execSQL(dropTableCmd);

	std::string createTableCmd = "CREATE TABLE test.asset"
		"("
		"inst_id integer NOT NULL,"
		"widget_id integer NOT NULL"
		");";
	dbExec->execSQL(createTableCmd);
	dbExec->execSQL("INSERT INTO test.asset VALUES (1, 4)");
	dbExec->execSQL("INSERT INTO test.asset VALUES (0, 5)");
	dbExec->execSQL("INSERT INTO test.asset VALUES (2, 3)");

	asset_tradable_combination_table tbl;

	dbExec->execSQLGet(tbl, "SELECT * from test.asset");
	
	dbExec->sortRows<0>(tbl, CDBExecutor::SORTORDER_DESC);
	BOOST_CHECK_EQUAL( (*tbl.begin())->get_inst_id(), 2);	
	BOOST_CHECK_EQUAL( tbl.back().get_inst_id(), 0);	

	dbExec->sortRows<1>(tbl);
	BOOST_CHECK_EQUAL( (*tbl.begin())->get_widget_id(), 3);	
	BOOST_CHECK_EQUAL( tbl.back().get_widget_id(), 5);	
};
BOOST_AUTO_TEST_SUITE_END()
